; GIMP - The GNU Image Manipulation Program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
;
; 
; Create an animated lava effect.  
;
; The script can be found on an open image in GIMP 2.4 under: Filters > Animation > Lava...
; Under GIMP 2.2, it can be found under: Filters > Animation > Animators > Lava...

; The animation may be saved with the gif-plug-in.
;
; The script lets you choose the image dimensions, number of frames in the animation
; the speed of the animation, the blend mode of the upper layer, whether or not
; you add the noise interactively (creates the lava) or let the script automatically do it
; for you and the lava color.

; Define the Function

(define (script-fu-lava-anim 	img		 
				drawable
				width
				height
				counter
				inSpeed
				blendOpt
				interactive
				color
				)

; Declare the Variables

	(let* (
		    	(newImage nil)		    
			(bottomCloudLayer nil)
			(bottomCloudLayerCopy nil)
			(topCloudLayer nil)
			(topCloudLayerCopy nil)
			(layerBlendMode
				(cond 
				(( equal? blendOpt 0 ) ADDITION-MODE)
				(( equal? blendOpt 1 ) DIVIDE-MODE)
				(( equal? blendOpt 2 ) SCREEN-MODE)
				(( equal? blendOpt 3 ) OVERLAY-MODE)
				(( equal? blendOpt 4 ) DODGE-MODE)
				(( equal? blendOpt 5 ) HARDLIGHT-MODE)
				(( equal? blendOpt 6 ) SOFTLIGHT-MODE)
				(( equal? blendOpt 7 ) GRAIN-EXTRACT-MODE)
				(( equal? blendOpt 8 ) GRAIN-MERGE-MODE)
				(( equal? blendOpt 9 ) DIFFERENCE-MODE)
				(( equal? blendOpt 10 ) MULTIPLY-MODE)
				(( equal? blendOpt 11 ) SUBTRACT-MODE)
				(( equal? blendOpt 12 ) DARKEN-ONLY-MODE)
				(( equal? blendOpt 13 ) LIGHTEN-ONLY-MODE)
								)	
			)
			(layerName nil)
			(runMode nil)
			(offset 0)
			(step nil)
			(frameNum 1)
	            
		)

(gimp-context-push)

(set! newImage (car (gimp-image-new width height RGB)))
(gimp-image-undo-disable newImage)
(set! bottomCloudLayer (car (gimp-layer-new newImage width height RGBA-IMAGE 	"Bottom Cloud Layer" 100 NORMAL-MODE)))
(gimp-image-add-layer newImage bottomCloudLayer -1)

(if (= interactive TRUE)
	(set! runMode RUN-INTERACTIVE)
	(set! runMode RUN-NONINTERACTIVE)
)

(plug-in-solid-noise runMode newImage bottomCloudLayer 1 1 (rand) 1 2.0 2.0)
(set! topCloudLayer (car (gimp-layer-copy bottomCloudLayer TRUE)))
(gimp-image-add-layer newImage topCloudLayer -1)
(plug-in-solid-noise runMode newImage topCloudLayer 1 1 (rand) 1 2.0 2.0)

(set! step (* -1 (/ height counter)))

(while (> counter 0)

(set! bottomCloudLayerCopy (car (gimp-layer-copy bottomCloudLayer TRUE)))
(gimp-image-add-layer newImage bottomCloudLayerCopy -1)
(set! layerName (string-append "Frame " (number->string frameNum) " (" (number->string inSpeed) "ms)" " (replace)"))
(gimp-drawable-set-name bottomCloudLayerCopy layerName)
(gimp-drawable-offset bottomCloudLayerCopy TRUE OFFSET-BACKGROUND 0.0 offset)
(set! topCloudLayerCopy (car (gimp-layer-copy topCloudLayer TRUE)))
(gimp-image-add-layer newImage topCloudLayerCopy -1)
(gimp-drawable-offset topCloudLayerCopy TRUE OFFSET-BACKGROUND 0.0 (* -1 offset))
(gimp-layer-set-mode topCloudLayerCopy layerBlendMode)
(gimp-image-merge-down newImage topCloudLayerCopy CLIP-TO-IMAGE)
(set! bottomCloudLayerCopy (car (gimp-layer-copy bottomCloudLayer TRUE)))
(gimp-image-add-layer newImage bottomCloudLayerCopy -1)
(gimp-context-set-foreground color)
(gimp-edit-fill bottomCloudLayerCopy FOREGROUND-FILL)
(gimp-layer-set-mode bottomCloudLayerCopy COLOR-MODE)
(gimp-image-merge-down newImage bottomCloudLayerCopy CLIP-TO-IMAGE)

(set! offset (- offset step))
(set! frameNum (+ frameNum 1))
(set! counter (- counter 1))

) ; goes with while

; Removes the original layer from the stack because it's no longer needed  
  
(gimp-image-remove-layer newImage bottomCloudLayer) 
(gimp-image-remove-layer newImage topCloudLayer) 
(gimp-image-undo-enable newImage)


(gimp-context-pop)

(gimp-displays-flush)

; Displays the final animation

(gimp-display-new newImage)

  )
)

(script-fu-register "script-fu-lava-anim"
  "<Image>/Filters/Animation/Animators/Lava..."
  "Create an animated lava effect"
  "Art Wade"
  "Art Wade"
  "2008"
  "RGB*"
  SF-IMAGE       	"Image" 0
  SF-DRAWABLE    	"Drawable" 0
  SF-ADJUSTMENT	"Image Width" '(200 100 2000 1 10 0 1)
  SF-ADJUSTMENT	"Image Height" '(200 100 2000 1 10 0 1)
  SF-ADJUSTMENT 	"Number of frames" '(30 15 2000 1 10 0 1)
  SF-ADJUSTMENT 	"Animation Speed (in ms)" '(150 10 500 1 1 0 1)
  SF-OPTION		"Upper Layer Blend Mode" 		'("ADDITION"
							"DIVIDE"
							"SCREEN"
							"OVERLAY"
							"DODGE"
							"HARDLIGHT"
							"SOFTLIGHT"
							"GRAIN EXTRACT"
							"GRAIN MERGE"												"DIFFERENCE"
							"MULTIPLY"
							"SUBTRACT"
							"DARKEN ONLY"
							"LIGHTEN ONLY")
SF-TOGGLE		"Add Noise Interactively?"	FALSE
SF-COLOR	"Lava Color" 	'(0 186 255)
)


                         
